<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8" />
    <title>cattle_grid</title>
  </head>
  <body>
    <h1>cattle_grid</h1>
    <p>
      cattle_grid is an authentication middleware for Fediverse applications.
      For more details see
      <a href="https://bovine.codeberg.page/cattle_grid/">cattle_grid</a>.
    </p>
  </body>
</html>
